@@ -128,13 +128,12 @@ static void capture_queue_cleanup(void* p) {
 	/*
 	 Currently, the only supported key is kCVPixelBufferPixelFormatTypeKey. Supported pixel formats are kCVPixelFormatType_420YpCbCr8BiPlanarVideoRange, kCVPixelFormatType_420YpCbCr8BiPlanarFullRange and kCVPixelFormatType_32BGRA, except on iPhone 3G, where the supported pixel formats are kCVPixelFormatType_422YpCbCr8 and kCVPixelFormatType_32BGRA..
 	 */
-	NSDictionary* dic = [NSDictionary dictionaryWithObjectsAndKeys:
-						 [NSNumber numberWithInteger:kCVPixelFormatType_420YpCbCr8BiPlanarVideoRange], (id)kCVPixelBufferPixelFormatTypeKey, nil];
+	NSDictionary* dic = @{ (id)kCVPixelBufferPixelFormatTypeKey : @(kCVPixelFormatType_420YpCbCr8BiPlanarVideoRange) };
 	[output setVideoSettings:dic];
 	
 	/* Set the layer */
 	AVCaptureVideoPreviewLayer *previewLayer = (AVCaptureVideoPreviewLayer *)self.layer;
-	[previewLayer setOrientation:AVCaptureVideoOrientationPortrait];
+	[previewLayer.connection setVideoOrientation:AVCaptureVideoOrientationPortrait];
 	[previewLayer setBackgroundColor:[[UIColor clearColor] CGColor]];
 	[previewLayer setOpaque:YES];
 	start_time=0;
@@ -163,7 +162,7 @@ static void capture_queue_cleanup(void* p) {
 	CVImageBufferRef frame = nil;
 	@synchronized(self) { 
 		@try {
-			CVImageBufferRef frame = CMSampleBufferGetImageBuffer(sampleBuffer); 
+			frame = CMSampleBufferGetImageBuffer(sampleBuffer);
 			CVReturn status = CVPixelBufferLockBaseAddress(frame, 0);
 			if (kCVReturnSuccess != status) {
 				ms_error("Error locking base address: %i", status);
@@ -236,7 +235,7 @@ static void capture_queue_cleanup(void* p) {
 				case 0:
 				case 180:
 					if (mOutputVideoSize.width*factor>plane_width || mOutputVideoSize.height*factor>plane_height) {
-						ms_warning("[1]IOS capture discarding frame because wrong dimensions (%d > %d || %d > %d)",
+						ms_warning("[1]IOS capture discarding frame because wrong dimensions (%d > %zu || %d > %zu)",
 								   mOutputVideoSize.width*factor, plane_width,
 								   mOutputVideoSize.height*factor, plane_height);
 						return;
@@ -245,7 +244,7 @@ static void capture_queue_cleanup(void* p) {
 				case 90:
 				case 270:
 					if (mOutputVideoSize.width*factor>plane_height || mOutputVideoSize.height*factor>plane_width) {
-						ms_warning("[2]	IOS capture discarding frame because wrong dimensions (%d > %d || %d > %d)",
+						ms_warning("[2]	IOS capture discarding frame because wrong dimensions (%d > %zu || %d > %zu)",
 								   mOutputVideoSize.width*factor, plane_height,
 								   mOutputVideoSize.height*factor, plane_width);
 						return;
@@ -260,8 +259,8 @@ static void capture_queue_cleanup(void* p) {
 																								   , rotation
 																								   , mOutputVideoSize.width
 																								   , mOutputVideoSize.height
-																								   , y_bytePer_row
-																								   , cbcr_bytePer_row
+																								   , (unsigned int)y_bytePer_row
+																								   , (unsigned int)cbcr_bytePer_row
 																								   , TRUE
 																								   , mDownScalingRequired); 
 			  
@@ -277,31 +276,39 @@ static void capture_queue_cleanup(void* p) {
 	}
 }
 
-- (void)openDevice:(const char*) deviceId {
+- (void)openDevice:(const char*) device_Id {
 	NSError *error = nil;
 	unsigned int i = 0;
 	AVCaptureDevice * device = NULL;
-	self->deviceId = deviceId;
+	self->deviceId = device_Id;
 	
 	NSArray * array = [AVCaptureDevice devicesWithMediaType:AVMediaTypeVideo];
 	for (i = 0 ; i < [array count]; i++) {
 		AVCaptureDevice * currentDevice = [array objectAtIndex:i];
-		if(!strcmp([[currentDevice uniqueID] UTF8String], deviceId)) {
+		if(!strcmp([[currentDevice uniqueID] UTF8String], device_Id)) {
 			device = currentDevice;
 			break;
 		}
 	}
 	if (device == NULL) {
-		ms_error("Error: camera %s not found, using default one", deviceId);
+		ms_error("Error: camera %s not found, using default one", device_Id);
 		device = [AVCaptureDevice defaultDeviceWithMediaType:AVMediaTypeVideo];
 	}
 	input = [AVCaptureDeviceInput deviceInputWithDevice:device
 												  error:&error];
-	[input retain]; // keep reference on an externally allocated object
-	
+
 	AVCaptureSession *session = [(AVCaptureVideoPreviewLayer *)self.layer session];
-	[session addInput:input];
-	[session addOutput:output];
+	if ( input && [session canAddInput:input] ){
+		[input retain]; // keep reference on an externally allocated object
+		[session addInput:input];
+	} else {
+		ms_error("Error: input nil or cannot be added: %p", input);
+	}
+	if( output && [session canAddOutput:output] ){
+		[session addOutput:output];
+	} else {
+		ms_error("Error: output nil or cannot be added: %p", output);
+	}
 }
 
 - (void)dealloc {
@@ -395,6 +402,16 @@ static AVCaptureVideoOrientation Angle2AVCaptureVideoOrientation(int deviceOrien
 		mCameraVideoSize = MS_VIDEO_SIZE_VGA;
 		mOutputVideoSize = MS_VIDEO_SIZE_QVGA;
 		mDownScalingRequired = true;
+	} else if ( (outputSize.width * outputSize.height) == (MS_VIDEO_SIZE_CIF_W * MS_VIDEO_SIZE_CIF_H) ){
+		[session setSessionPreset:AVCaptureSessionPreset352x288];
+        mCameraVideoSize     = MS_VIDEO_SIZE_CIF;
+        mOutputVideoSize     = MS_VIDEO_SIZE_CIF;
+        mDownScalingRequired = FALSE;
+	} else if ( (outputSize.width * outputSize.height) == (MS_VIDEO_SIZE_QCIF_W * MS_VIDEO_SIZE_QCIF_H) ){
+		[session setSessionPreset:AVCaptureSessionPreset352x288];
+        mCameraVideoSize     = MS_VIDEO_SIZE_CIF;
+        mOutputVideoSize     = MS_VIDEO_SIZE_QCIF;
+        mDownScalingRequired = TRUE;
 	} else {
 		// Default case
 		[session setSessionPreset: AVCaptureSessionPresetMedium];
@@ -422,6 +439,10 @@ static AVCaptureVideoOrientation Angle2AVCaptureVideoOrientation(int deviceOrien
 			vsize = MS_VIDEO_SIZE_VGA;
 		} else if ((size.width * size.height) == (MS_VIDEO_SIZE_QVGA_W * MS_VIDEO_SIZE_QVGA_H)) {
 			vsize = MS_VIDEO_SIZE_QVGA;
+		} else if ( (size.width * size.height) == (MS_VIDEO_SIZE_CIF_W * MS_VIDEO_SIZE_CIF_H )) {
+			vsize = MS_VIDEO_SIZE_CIF;
+		} else if ( (size.width * size.height) == (MS_VIDEO_SIZE_QCIF_W * MS_VIDEO_SIZE_QCIF_H) ) {
+			vsize = MS_VIDEO_SIZE_QCIF;
 		}
 		[self configureSize:vsize withSession:session];
 		
@@ -468,16 +489,32 @@ static AVCaptureVideoOrientation Angle2AVCaptureVideoOrientation(int deviceOrien
 	@synchronized(self) {
 		AVCaptureSession *session = [(AVCaptureVideoPreviewLayer *)self.layer session];
 		[session beginConfiguration];
-		if ([[[UIDevice currentDevice] systemVersion] floatValue] < 5) { 
-			[output setMinFrameDuration:CMTimeMake(1, value)];
+
+		if( [[[UIDevice currentDevice] systemVersion] floatValue] >= 7 ){
+			for( AVCaptureDeviceInput* devinput in [session inputs] ) {
+
+				NSError* err = nil;
+				if( [devinput.device lockForConfiguration:&err] == YES ){
+
+					[devinput.device setActiveVideoMinFrameDuration:CMTimeMake(1, value)];
+					[devinput.device setActiveVideoMaxFrameDuration:CMTimeMake(1, value)];
+
+					[devinput.device unlockForConfiguration];
+				} else {
+					ms_error("Couldn't obtain lock to set capture FPS: %s", err?[err.description UTF8String] : "");
+				}
+
+				break;
+			}
 		} else {
+			// Pre-iOS7 method
 			NSArray *connections = output.connections;
 			if ([connections count] > 0) {
 				[[connections objectAtIndex:0] setVideoMinFrameDuration:CMTimeMake(1, value)];
 				[[connections objectAtIndex:0] setVideoMaxFrameDuration:CMTimeMake(1, value)];
 			} 
-			
 		}
+
 		fps=value;
 		snprintf(fps_context, sizeof(fps_context), "Captured mean fps=%%f, expected=%f", fps);
 		ms_video_init_average_fps(&averageFps, fps_context);
@@ -543,7 +580,7 @@ static void ioscapture_process(MSFilter * obj) {
 			// keep only the latest image
 			ms_queue_flush(obj->outputs[0]);
 			ms_queue_put(obj->outputs[0],thiz->msframe);
-			ms_video_update_average_fps(&thiz->averageFps, obj->ticker->time);
+			ms_video_update_average_fps(&thiz->averageFps, (uint32_t)obj->ticker->time);
 			thiz->msframe=0;
 		}	
 		ms_mutex_unlock(&thiz->mutex);
@@ -565,7 +602,7 @@ static void ioscapture_postprocess(MSFilter *f) {
 static int ioscapture_get_fps(MSFilter *f, void *arg) {
 	IOSCapture *thiz = (IOSCapture*)f->data;
 	if (thiz != NULL) {
-		*((float*)arg) = thiz->fps;
+		*((float*)arg) = ms_average_fps_get(&thiz->averageFps);
 	}
 	return 0;
 }
@@ -643,8 +680,8 @@ static int ioscapture_set_device_orientation_display (MSFilter *f, void *arg) {
 	IOSCapture *thiz=(IOSCapture*)f->data;
 	if (thiz != NULL) {
 		AVCaptureVideoPreviewLayer *previewLayer = (AVCaptureVideoPreviewLayer *)thiz.layer;
-		if ([previewLayer isOrientationSupported])
-			previewLayer.orientation = Angle2AVCaptureVideoOrientation(*(int*)(arg));
+		if ([previewLayer.connection isVideoOrientationSupported])
+			previewLayer.connection.videoOrientation = Angle2AVCaptureVideoOrientation(*(int*)(arg));
 	}
 	return 0;
 }
